# Copyright 2010-2011 OpenStack Foundation
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import warnings

from oslotest import base


class TestCase(base.BaseTestCase):

    """Test case base class for all unit tests."""

    def setUp(self):
        super().setUp()
        # FIXME(stephenfin): Determine if we need to replace use of best_match
        warnings.filterwarnings(
            'ignore',
            module='webob',
            message='The behavior of AcceptValidHeader.best_match is ',
            category=DeprecationWarning,
        )

        # FIXME(stephenfin): Determine if we need to replace use of best_match
        warnings.filterwarnings(
            'ignore',
            module='webob',
            message='The behavior of .best_match for the Accept classes is ',
            category=DeprecationWarning,
        )



#
# Copyright 2025 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from aetos.controllers.api.v1.admin import root as admin_root
from aetos.controllers.api.v1 import label
from aetos.controllers.api.v1 import labels
from aetos.controllers.api.v1 import query
from aetos.controllers.api.v1 import series
from aetos.controllers.api.v1 import status
from aetos.controllers.api.v1 import targets


class V1Controller:
    """Version 1 API controller root."""

    query = query.QueryController()
    series = series.SeriesController()
    labels = labels.LabelsController()
    label = label.LabelController()
    targets = targets.TargetsController()
    status = status.StatusController()
    admin = admin_root.AdminController()



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_config import cfg
from oslo_policy import policy

RULE_CONTEXT_IS_ADMIN = 'rule:context_is_admin'
RULE_ADMIN_OR_OWNER = 'rule:context_is_admin or project_id:%(project_id)s'
UNPROTECTED = ''

# Constants that represent common personas.
PROJECT_ADMIN = 'role:admin and project_id:%(project_id)s'
PROJECT_MEMBER = 'role:member and project_id:%(project_id)s'
PROJECT_READER = 'role:reader and project_id:%(project_id)s'
SERVICE = 'role:service'
PROJECT_ADMIN_OR_SERVICE = f'({PROJECT_ADMIN}) or ({SERVICE})'

rules = [
    policy.RuleDefault(
        name="segregation",
        check_str=RULE_CONTEXT_IS_ADMIN),

    policy.DocumentedRuleDefault(
        name="telemetry:admin_delete_metrics",
        check_str=PROJECT_ADMIN,
        scope_types=['project'],
        description='Delete metrics.',
        operations=[
            {
                'path': '/api/v1/admin/tsdb/delete_series',
                'method': 'POST'
            }
        ],
    ),
    policy.DocumentedRuleDefault(
        name="telemetry:admin_snapshot",
        check_str=PROJECT_ADMIN,
        scope_types=['project'],
        description='Take snapshot of the database.',
        operations=[
            {
                'path': '/api/v1/admin/tsdb/snapshot',
                'method': 'POST'
            }
        ],
    ),
    policy.DocumentedRuleDefault(
        name="telemetry:admin_clean_tombstones",
        check_str=PROJECT_ADMIN,
        scope_types=['project'],
        description='Clean tombstones.',
        operations=[
            {
                'path': '/api/v1/admin/tsdb/clean_tombstones',
                'method': 'POST'
            }
        ],
    ),
    policy.DocumentedRuleDefault(
        name="telemetry:query",
        check_str=PROJECT_READER,
        scope_types=['project'],
        description='Prometheus Query endpoint with tenancy enforced.',
        operations=[
            {
                'path': '/api/v1/query',
                'method': 'GET'
            }
        ],
    ),
    policy.DocumentedRuleDefault(
        name="telemetry:query:all_projects",
        check_str=PROJECT_ADMIN_OR_SERVICE,
        scope_types=['project'],
        description='Prometheus Query endpoint without tenancy enforced.',
        operations=[
            {
                'path': '/api/v1/query',
                'method': 'GET'
            }
        ],
    ),
    policy.DocumentedRuleDefault(
        name="telemetry:label",
        check_str=PROJECT_READER,
        scope_types=['project'],
        description='Prometheus label endpoint with tenancy enforced.',
        operations=[
            {
                'path': '/api/v1/label',
                'method': 'GET'
            }
        ],
    ),
    policy.DocumentedRuleDefault(
        name="telemetry:label:all_projects",
        check_str=PROJECT_ADMIN_OR_SERVICE,
        scope_types=['project'],
        description='Prometheus label endpoint without tenancy enforced.',
        operations=[
            {
                'path': '/api/v1/label',
                'method': 'GET'
            }
        ],
    ),
    policy.DocumentedRuleDefault(
        name="telemetry:labels",
        check_str=PROJECT_READER,
        scope_types=['project'],
        description='Prometheus labels endpoint with tenancy enforced.',
        operations=[
            {
                'path': '/api/v1/labels',
                'method': 'GET'
            }
        ],
    ),
    policy.DocumentedRuleDefault(
        name="telemetry:labels:all_projects",
        check_str=PROJECT_ADMIN_OR_SERVICE,
        scope_types=['project'],
        description='Prometheus labels endpoint without tenancy enforced.',
        operations=[
            {
                'path': '/api/v1/labels',
                'method': 'GET'
            }
        ],
    ),
    policy.DocumentedRuleDefault(
        name="telemetry:series",
        check_str=PROJECT_READER,
        scope_types=['project'],
        description='Prometheus series endpoint with tenancy enforced.',
        operations=[
            {
                'path': '/api/v1/series',
                'method': 'GET'
            }
        ],
    ),
    policy.DocumentedRuleDefault(
        name="telemetry:series:all_projects",
        check_str=PROJECT_ADMIN_OR_SERVICE,
        scope_types=['project'],
        description='Prometheus series endpoint without tenancy enforced.',
        operations=[
            {
                'path': '/api/v1/series',
                'method': 'GET'
            }
        ],
    ),
    policy.DocumentedRuleDefault(
        name="telemetry:targets",
        check_str=PROJECT_ADMIN_OR_SERVICE,
        scope_types=['project'],
        description='Prometheus targets endpoint.',
        operations=[
            {
                'path': '/api/v1/targets',
                'method': 'GET'
            }
        ],
    ),
    policy.DocumentedRuleDefault(
        name="telemetry:status",
        check_str=PROJECT_ADMIN_OR_SERVICE,
        scope_types=['project'],
        description='Prometheus status endpoint.',
        operations=[
            {
                'path': '/api/v1/status',
                'method': 'GET'
            }
        ],
    ),
]


def list_rules():
    return rules


def init(conf):
    enforcer = policy.Enforcer(conf, default_rule="default")
    enforcer.register_defaults(list_rules())
    return enforcer


def get_enforcer():
    # This method is used by oslopolicy CLI scripts in order to generate policy
    # files from overrides on disk and defaults in code.
    cfg.CONF([], project='aetos')
    return init(cfg.CONF)



#
# Copyright 2013-2025 Red Hat, Inc
# Copyright 2012-2015 eNovance <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os

from oslo_config import cfg
import oslo_i18n
from oslo_log import log
from oslo_policy import opts as policy_opts

from aetos.conf import defaults
from aetos import version


def prepare_service(argv=None, config_files=None):
    conf = cfg.ConfigOpts()
    oslo_i18n.enable_lazy()
    log.register_options(conf)
    defaults.set_cors_middleware_defaults()
    policy_opts.set_defaults(conf, policy_file=os.path.abspath(
        os.path.join(os.path.dirname(__file__), "api", "policy.yaml")))
    from aetos import opts
    # Register our own Aetos options
    for group, options in opts.list_opts():
        conf.register_opts(list(options),
                           group=None if group == "DEFAULT" else group)

    conf(argv, project='aetos', validate_default_values=True,
         default_config_files=config_files,
         version=version.version_info.version_string())

    log.setup(conf, 'aetos')

    return conf



#
# Copyright 2025 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslo_config import fixture as fixture_config

from aetos.controllers.api.v1 import base
from aetos.tests import base as test_base


class TestBaseController(test_base.TestCase):

    def setUp(self):
        super().setUp()
        self.controller = base.Base()
        self.conf = self.useFixture(fixture_config.Config()).conf
        self.conf.register_opts(base.PROMETHEUS_OPTS, group='prometheus')

    def test_create_prometheus_client_without_tls(self):
        """Test prometheus client creation without TLS"""
        with mock.patch(
            'observabilityclient.prometheus_client.PrometheusAPIClient'
        ) as mock_client:
            self.controller.create_prometheus_client(self.conf)

            mock_client.assert_called_once_with('localhost:9090')
            mock_client.return_value.set_ca_cert.assert_not_called()
            mock_client.return_value.set_client_cert.assert_not_called()

    def test_create_prometheus_client_with_tls_custom_ca(self):
        """Test prometheus client creation with TLS with custom CA"""
        self.conf.set_override('ca_file', 'cafile', group='prometheus')
        self.conf.set_override('use_tls', True, group='prometheus')

        with mock.patch(
            'observabilityclient.prometheus_client.PrometheusAPIClient'
        ) as mock_client:
            self.controller.create_prometheus_client(self.conf)

            mock_client.assert_called_once_with('localhost:9090')
            mock_client.return_value.set_ca_cert.assert_called_once_with(
                'cafile')
            mock_client.return_value.set_client_cert.assert_not_called()

    def test_create_prometheus_client_with_tls_default_ca(self):
        """Test prometheus client creation with TLS with default CA"""
        self.conf.set_override('use_tls', True, group='prometheus')

        with mock.patch(
            'observabilityclient.prometheus_client.PrometheusAPIClient'
        ) as mock_client:
            self.controller.create_prometheus_client(self.conf)

            mock_client.assert_called_once_with('localhost:9090')
            mock_client.return_value.set_ca_cert.assert_called_once_with(True)
            mock_client.return_value.set_client_cert.assert_not_called()

    def test_create_prometheus_client_without_tls_with_ca(self):
        """Test prometheus client creation without TLS but with CA"""
        self.conf.set_override('ca_file', 'cafile', group='prometheus')
        with mock.patch(
            'observabilityclient.prometheus_client.PrometheusAPIClient'
        ) as mock_client:
            self.controller.create_prometheus_client(self.conf)

            mock_client.assert_called_once_with('localhost:9090')
            mock_client.return_value.set_ca_cert.assert_not_called()
            mock_client.return_value.set_client_cert.assert_not_called()

    def test_create_prometheus_client_with_tls_with_cert_and_key(self):
        """Test prometheus client creation with TLS with cert/key file"""
        self.conf.set_override('use_tls', True, group='prometheus')
        self.conf.set_override('cert_file', 'certfile', group='prometheus')
        self.conf.set_override('key_file', 'keyfile', group='prometheus')

        with mock.patch(
            'observabilityclient.prometheus_client.PrometheusAPIClient'
        ) as mock_client:
            self.controller.create_prometheus_client(self.conf)

            mock_client.assert_called_once_with('localhost:9090')
            mock_client.return_value.set_ca_cert.assert_called_once_with(True)
            mock_client.return_value.set_client_cert.assert_called_once_with(
                'certfile', 'keyfile')

    def test_create_prometheus_client_without_tls_with_cert_and_key(self):
        """Test prometheus client creation without TLS with cert/key file"""
        self.conf.set_override('cert_file', 'certfile', group='prometheus')
        self.conf.set_override('key_file', 'keyfile', group='prometheus')

        with mock.patch(
            'observabilityclient.prometheus_client.PrometheusAPIClient'
        ) as mock_client:
            self.controller.create_prometheus_client(self.conf)

            mock_client.assert_called_once_with('localhost:9090')
            mock_client.return_value.set_ca_cert.assert_not_called()
            mock_client.return_value.set_client_cert.assert_not_called()

    def test_create_prometheus_client_with_tls_with_cert_only(self):
        """Test prometheus client creation with TLS with only cert file"""
        self.conf.set_override('use_tls', True, group='prometheus')
        self.conf.set_override('key_file', 'keyfile', group='prometheus')

        with mock.patch(
            'observabilityclient.prometheus_client.PrometheusAPIClient'
        ) as mock_client:
            self.controller.create_prometheus_client(self.conf)

            mock_client.assert_called_once_with('localhost:9090')
            mock_client.return_value.set_ca_cert.assert_called_once_with(True)
            mock_client.return_value.set_client_cert.assert_not_called()

    def test_create_prometheus_client_with_tls_with_key_only(self):
        """Test prometheus client creation with TLS with only ket file"""
        self.conf.set_override('use_tls', True, group='prometheus')
        self.conf.set_override('key_file', 'keyfile', group='prometheus')

        with mock.patch(
            'observabilityclient.prometheus_client.PrometheusAPIClient'
        ) as mock_client:
            self.controller.create_prometheus_client(self.conf)

            mock_client.assert_called_once_with('localhost:9090')
            mock_client.return_value.set_ca_cert.assert_called_once_with(True)
            mock_client.return_value.set_client_cert.assert_not_called()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_core_endpoints
----------------------------------

Tests for endpoints under /api/v1
"""

from observabilityclient import prometheus_client
from observabilityclient import rbac
import os
from unittest import mock
import webtest

from aetos import app
from aetos.tests.functional import base


class TestCoreEndpointsForbidden(base.TestCase):
    def setUp(self):
        super().setUp()
        self.expected_status_code = 403
        self.expected_fault_string = "RBAC Authorization Failed"

        pf = os.path.abspath('aetos/tests/functional/policy.yaml-test')
        self.CONF.set_override('policy_file', pf, group='oslo_policy')
        self.CONF.set_override('auth_mode', None, group=None)
        self.app = webtest.TestApp(app.load_app(self.CONF))

    def test_label(self):
        label_name = 'job'

        result = self.get_json(f'/label/{label_name}/values', {},
                               headers=self.reader_auth_headers,
                               status=self.expected_status_code)

        self.assertEqual(self.expected_status_code, result.status_code)
        # NOTE(jwysogla): the series doesn't use wsme, so the
        # error message is slightly differently formatted, but the same
        # status code is still returned and the meaning of the error message
        # is also the same. But this is the reason why we're using assertIn
        # instead of assertEqual here.
        self.assertIn(self.expected_fault_string, result.json['error_message'])

    def test_labels(self):
        result = self.get_json('/labels', {},
                               headers=self.reader_auth_headers,
                               status=self.expected_status_code)

        self.assertEqual(self.expected_status_code, result.status_code)
        # NOTE(jwysogla): the series doesn't use wsme, so the
        # error message is slightly differently formatted, but the same
        # status code is still returned and the meaning of the error message
        # is also the same. But this is the reason why we're using assertIn
        # instead of assertEqual here.
        self.assertIn(self.expected_fault_string, result.json['error_message'])

    def test_query(self):
        query_string = 'ceilometer_image_size'
        params = {'query': query_string}

        result = self.get_json('/query', **params,
                               headers=self.reader_auth_headers,
                               status=self.expected_status_code)

        self.assertEqual(self.expected_status_code, result.status_code)
        self.assertEqual(self.expected_fault_string,
                         result.json['error_message']['faultstring'])

    def test_series(self):
        match = 'ceilometer_image_size'
        params = {'match[]': match}

        result = self.get_json('/series', **params,
                               headers=self.reader_auth_headers,
                               status=self.expected_status_code)

        self.assertEqual(self.expected_status_code, result.status_code)
        # NOTE(jwysogla): the series doesn't use wsme, so the
        # error message is slightly differently formatted, but the same
        # status code is still returned and the meaning of the error message
        # is also the same. But this is the reason why we're using assertIn
        # instead of assertEqual here.
        self.assertIn(self.expected_fault_string, result.json['error_message'])

    def test_status(self):
        result = self.get_json('/status/runtimeinfo', {},
                               headers=self.reader_auth_headers,
                               status=self.expected_status_code)

        self.assertEqual(self.expected_status_code, result.status_code)
        self.assertEqual(self.expected_fault_string,
                         result.json['error_message']['faultstring'])

    def test_targets(self):
        params = {'state': 'running'}
        result = self.get_json('/targets', **params,
                               headers=self.reader_auth_headers,
                               status=self.expected_status_code)

        self.assertEqual(self.expected_status_code, result.status_code)
        self.assertEqual(self.expected_fault_string,
                         result.json['error_message']['faultstring'])


class TestCoreEndpointsAsUser(base.TestCase):
    def test_label(self):
        self.CONF.set_override(
            'project_label_name', 'some_other_label', group='prometheus')
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": [
                "prometheus",
                "sg-core"
                ]
            }

        label_name = 'job'
        expected_match = f"{{some_other_label='{self.project_id}'}}"
        expected_params = {'match[]': expected_match}

        with (
            mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                              return_value=returned_from_prometheus
                              ) as get_mock,
            mock.patch.object(rbac.PromQLRbac, 'append_rbac_labels',
                              return_value=expected_match) as rbac_mock,
            mock.patch.object(rbac.PromQLRbac, '__init__',
                              return_value=None
                              ) as rbac_init_mock
            ):
            result = self.get_json(f'/label/{label_name}/values', {},
                                   headers=self.reader_auth_headers,
                                   status=expected_status_code)

        self.assertEqual(returned_from_prometheus, result.json)
        self.assertEqual(expected_status_code, result.status_code)
        get_mock.assert_called_once_with(
            f'label/{label_name}/values', expected_params
        )
        rbac_mock.assert_called_once_with('')
        rbac_init_mock.assert_called_once()
        self.assertEqual('some_other_label', rbac_init_mock.call_args.args[2])

    def test_label_with_matches(self):
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": [
                "prometheus",
                ]
            }

        label_name = 'job'
        matches = ["ceilometer_cpu", "{job='prometheus'}"]
        params = {'match[]': matches}
        expected_matches = [
            f"ceilometer_cpu{{project='{self.project_id}'}}",
            f"{{job='prometheus', project='{self.project_id}'}}"
        ]
        expected_params = {'match[]': expected_matches}

        with (
            mock.patch.object(prometheus_client.PrometheusAPIClient,
                              'label_values', return_value=['metric_name']),
            mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                              return_value=returned_from_prometheus
                              ) as get_mock,
            mock.patch.object(rbac.PromQLRbac, 'modify_query',
                              side_effect=lambda x, metric_names:
                              expected_matches[matches.index(x)]
                              ) as rbac_mock
            ):
            result = self.get_json(f'/label/{label_name}/values', **params,
                                   headers=self.reader_auth_headers,
                                   status=expected_status_code)

        self.assertEqual(returned_from_prometheus, result.json)
        self.assertEqual(expected_status_code, result.status_code)
        get_mock.assert_called_once_with(
            f'label/{label_name}/values', expected_params
        )
        for match in matches:
            rbac_mock.assert_any_call(match, metric_names=['metric_name'])

    def test_labels(self):
        self.CONF.set_override(
            'project_label_name', 'some_other_label', group='prometheus')
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": [
                "__name__",
                "alarm",
                "branch",
                "volume"
                ]
            }

        expected_match = f"{{some_other_label='{self.project_id}'}}"
        expected_params = {'match[]': expected_match}

        with (
            mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                              return_value=returned_from_prometheus
                              ) as get_mock,
            mock.patch.object(rbac.PromQLRbac, 'append_rbac_labels',
                              return_value=expected_match) as rbac_mock,
            mock.patch.object(rbac.PromQLRbac, '__init__',
                              return_value=None
                              ) as rbac_init_mock
            ):
            result = self.get_json('/labels', {},
                                   headers=self.reader_auth_headers,
                                   status=expected_status_code)

        self.assertEqual(returned_from_prometheus, result.json)
        self.assertEqual(expected_status_code, result.status_code)
        get_mock.assert_called_once_with(
            'labels', expected_params
        )
        rbac_mock.assert_called_once_with('')
        rbac_init_mock.assert_called_once()
        self.assertEqual('some_other_label', rbac_init_mock.call_args.args[2])

    def test_labels_with_matches(self):
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": [
                "__name__",
                "alarm",
                "branch",
                "volume"
                ]
            }

        matches = ["ceilometer_cpu", "{job='prometheus'}"]
        params = {'match[]': matches}
        expected_matches = [
            f"ceilometer_cpu{{project='{self.project_id}'}}",
            f"{{job='prometheus', project='{self.project_id}'}}"
        ]
        expected_params = {'match[]': expected_matches}

        with (
            mock.patch.object(prometheus_client.PrometheusAPIClient,
                              'label_values', return_value=['metric_name']),
            mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                              return_value=returned_from_prometheus
                              ) as get_mock,
            mock.patch.object(rbac.PromQLRbac, 'modify_query',
                              side_effect=lambda x, metric_names:
                              expected_matches[matches.index(x)]
                              ) as rbac_mock
            ):
            result = self.get_json('/labels', **params,
                                   headers=self.reader_auth_headers,
                                   status=expected_status_code)

        self.assertEqual(returned_from_prometheus, result.json)
        self.assertEqual(expected_status_code, result.status_code)
        get_mock.assert_called_once_with(
            'labels', expected_params
        )
        for match in matches:
            rbac_mock.assert_any_call(match, metric_names=['metric_name'])

    def test_query(self):
        self.CONF.set_override(
            'project_label_name', 'some_other_label', group='prometheus')
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": {
                "resultType": "vector",
                "result": [
                    {
                        "metric": {
                            "__name__": "ceilometer_image_size",
                            "counter": "image.size",
                            "image": "828ab616-8904-48fb-a4bb-d037473cee7d",
                            "instance": "localhost:3000",
                            "job": "sg-core",
                            "some_other_label":
                                "2dd8edd6c8c24f49bf04670534f6b357",
                            "publisher": "localhost.localdomain",
                            "resource": "828ab616-8904-48fb-a4bb-d037473cee7d",
                            "resource_name": "cirros-0.6.2-x86_64-disk",
                            "type": "size",
                            "unit": "B"
                            },
                        "value": [
                            1748273657.273,
                            "21430272"
                            ]
                        }
                    ]
                }
            }

        query_string = 'ceilometer_image_size'
        modified_query_string = \
            f'ceilometer_image_size{{some_other_label={self.project_id}}}'
        params = {
            'query': query_string,
            'time': '2026-02-23T13:20:05+00:00',
            'timeout': '200'
        }
        modified_params = {
            'query': modified_query_string,
            'time': '2026-02-23T13:20:05+00:00',
            'timeout': '200'
        }

        with (
            mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                              return_value=returned_from_prometheus
                              ) as get_mock,
            mock.patch.object(rbac.PromQLRbac, 'modify_query',
                              return_value=modified_query_string) as rbac_mock,
            mock.patch.object(rbac.PromQLRbac, '__init__',
                              return_value=None
                              ) as rbac_init_mock
            ):
            result = self.get_json('/query', **params,
                                   headers=self.reader_auth_headers,
                                   status=expected_status_code)

        self.assertEqual(returned_from_prometheus, result.json)
        self.assertEqual(expected_status_code, result.status_code)
        get_mock.assert_called_once_with('query', modified_params)
        rbac_mock.assert_called_once_with(query_string)
        rbac_init_mock.assert_called_once()
        self.assertEqual('some_other_label', rbac_init_mock.call_args.args[2])

    def test_series(self):
        self.CONF.set_override(
            'project_label_name', 'some_other_label', group='prometheus')
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": [
                {
                    "__name__": "ceilometer_image_size",
                    "counter": "image.size",
                    "image": "18f639e4-3d0c-447c-a81f-d00db66e63f3",
                    "instance": "localhost:3000",
                    "job": "sg-core",
                    "some_other_label": "7b8e1f013ad240fabe4ff2a4f44345fd",
                    "publisher": "localhost.localdomain",
                    "resource": "18f639e4-3d0c-447c-a81f-d00db66e63f3",
                    "resource_name": "tempest-scenario-img--2041421357",
                    "type": "size",
                    "unit": "B"
                    }
                ]
            }

        matches = ['ceilometer_image_size', '{resource="volume.size"}']
        modified_matches = [
            f'ceilometer_image_size{{some_other_label={self.project_id}}}',
            f'{{some_other_label={self.project_id}, resource="volume.size"}}'
        ]
        params = {'match[]': matches}
        modified_params = {'match[]': modified_matches}

        with (
            mock.patch.object(prometheus_client.PrometheusAPIClient,
                              'label_values', return_value=['metric_name']),
            mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                              return_value=returned_from_prometheus
                              ) as get_mock,
            mock.patch.object(rbac.PromQLRbac, 'modify_query',
                              side_effect=lambda x, metric_names:
                              modified_matches[matches.index(x)]
                              ) as rbac_mock,
            mock.patch.object(rbac.PromQLRbac, '__init__',
                              return_value=None
                              ) as rbac_init_mock
            ):
            result = self.get_json('/series', **params,
                                   headers=self.reader_auth_headers,
                                   status=expected_status_code)

        self.assertEqual(returned_from_prometheus, result.json)
        self.assertEqual(expected_status_code, result.status_code)
        get_mock.assert_called_once_with('series', modified_params)
        for match in matches:
            rbac_mock.assert_any_call(match, metric_names=['metric_name'])
        rbac_init_mock.assert_called_once()
        self.assertEqual('some_other_label', rbac_init_mock.call_args.args[2])

    def test_status(self):
        expected_status_code = 403
        expected_fault_string = "RBAC Authorization Failed"

        result = self.get_json('/status/runtimeinfo', {},
                               headers=self.reader_auth_headers,
                               status=expected_status_code)

        self.assertEqual(expected_status_code, result.status_code)
        self.assertEqual(expected_fault_string,
                         result.json['error_message']['faultstring'])

    def test_targets(self):
        expected_status_code = 403
        expected_fault_string = "RBAC Authorization Failed"
        params = {'state': 'running'}

        result = self.get_json('/targets', **params,
                               headers=self.reader_auth_headers,
                               status=expected_status_code)

        self.assertEqual(expected_status_code, result.status_code)
        self.assertEqual(expected_fault_string,
                         result.json['error_message']['faultstring'])

    def test_project_label_name_default(self):
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": [
                {
                    "__name__": "ceilometer_image_size",
                    "counter": "image.size",
                    "image": "18f639e4-3d0c-447c-a81f-d00db66e63f3",
                    "instance": "localhost:3000",
                    "job": "sg-core",
                    "project": "7b8e1f013ad240fabe4ff2a4f44345fd",
                    "publisher": "localhost.localdomain",
                    "resource": "18f639e4-3d0c-447c-a81f-d00db66e63f3",
                    "resource_name": "tempest-scenario-img--2041421357",
                    "type": "size",
                    "unit": "B"
                    }
                ]
            }

        matches = ['ceilometer_image_size', '{resource="volume.size"}']
        params = {'match[]': matches}

        with (
            mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                              return_value=returned_from_prometheus
                              ),
            mock.patch.object(rbac.PromQLRbac, 'modify_query'),
            mock.patch.object(rbac.PromQLRbac, '__init__',
                              return_value=None
                              ) as rbac_init_mock
            ):
            self.get_json('/series', **params,
                          headers=self.reader_auth_headers,
                          status=expected_status_code)
        rbac_init_mock.assert_called_once()
        self.assertEqual('project', rbac_init_mock.call_args.args[2])


class TestCoreEndpointsAsAdmin(base.TestCase):
    def test_label(self):
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": [
                "prometheus",
                "sg-core"
                ]
            }

        label_name = 'job'
        expected_params = {'match[]': []}

        with mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                               return_value=returned_from_prometheus
                               ) as get_mock:
            result = self.get_json(f'/label/{label_name}/values', {},
                                   headers=self.admin_auth_headers,
                                   status=expected_status_code)

        self.assertEqual(returned_from_prometheus, result.json)
        self.assertEqual(expected_status_code, result.status_code)
        get_mock.assert_called_once_with(
            f'label/{label_name}/values',
            expected_params
        )

    def test_labels(self):
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": [
                "__name__",
                "alarm",
                "branch",
                "volume"
                ]
            }
        expected_params = {'match[]': []}

        with mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                               return_value=returned_from_prometheus
                               ) as get_mock:
            result = self.get_json('/labels', {},
                                   headers=self.admin_auth_headers,
                                   status=expected_status_code)

        self.assertEqual(returned_from_prometheus, result.json)
        self.assertEqual(expected_status_code, result.status_code)
        get_mock.assert_called_once_with('labels', expected_params)

    def test_query(self):
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": {
                "resultType": "vector",
                "result": [
                    {
                        "metric": {
                            "__name__": "ceilometer_image_size",
                            "counter": "image.size",
                            "image": "828ab616-8904-48fb-a4bb-d037473cee7d",
                            "instance": "localhost:3000",
                            "job": "sg-core",
                            "project": "2dd8edd6c8c24f49bf04670534f6b357",
                            "publisher": "localhost.localdomain",
                            "resource": "828ab616-8904-48fb-a4bb-d037473cee7d",
                            "resource_name": "cirros-0.6.2-x86_64-disk",
                            "type": "size",
                            "unit": "B"
                            },
                        "value": [
                            1748273657.273,
                            "21430272"
                            ]
                        }
                    ]
                }
            }

        query_string = 'ceilometer_image_size'
        params = {'query': query_string}
        expected_params = {
            'query': query_string,
            'time': None,
            'timeout': None
        }

        with (
            mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                              return_value=returned_from_prometheus
                              ) as get_mock,
            mock.patch.object(rbac.PromQLRbac, 'modify_query') as rbac_mock
            ):
            result = self.get_json('/query', **params,
                                   headers=self.admin_auth_headers,
                                   status=expected_status_code)

        self.assertEqual(returned_from_prometheus, result.json)
        self.assertEqual(expected_status_code, result.status_code)
        get_mock.assert_called_once_with('query', expected_params)
        rbac_mock.assert_not_called()

    def test_series(self):
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": [
                {
                    "__name__": "ceilometer_image_size",
                    "counter": "image.size",
                    "image": "18f639e4-3d0c-447c-a81f-d00db66e63f3",
                    "instance": "localhost:3000",
                    "job": "sg-core",
                    "project": "7b8e1f013ad240fabe4ff2a4f44345fd",
                    "publisher": "localhost.localdomain",
                    "resource": "18f639e4-3d0c-447c-a81f-d00db66e63f3",
                    "resource_name": "tempest-scenario-img--2041421357",
                    "type": "size",
                    "unit": "B"
                    }
                ]
            }

        matches = ['ceilometer_image_size', '{resource="volume.size"}']
        params = {'match[]': matches}

        with (
            mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                              return_value=returned_from_prometheus
                              ) as get_mock,
            mock.patch.object(rbac.PromQLRbac, 'modify_query'
                              ) as rbac_mock
            ):
            result = self.get_json('/series', **params,
                                   headers=self.admin_auth_headers,
                                   status=expected_status_code)

        self.assertEqual(returned_from_prometheus, result.json)
        self.assertEqual(expected_status_code, result.status_code)
        get_mock.assert_called_once_with('series', params)
        rbac_mock.assert_not_called()

    def test_status(self):
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": {
                "startTime": "2025-05-26T15:32:23.890553181Z",
                "CWD": "/prometheus",
                "reloadConfigSuccess": True,
                "lastConfigTime": "2025-05-26T15:32:24Z",
                "corruptionCount": 0,
                "goroutineCount": 34,
                "GOMAXPROCS": 4,
                "GOMEMLIMIT": 9223372036854775807,
                "GOGC": "75",
                "GODEBUG": "",
                "storageRetention": "15d"
                }
            }

        with mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                               return_value=returned_from_prometheus
                               ) as get_mock:
            result = self.get_json('/status/runtimeinfo', {},
                                   headers=self.admin_auth_headers,
                                   status=expected_status_code)

        self.assertEqual(returned_from_prometheus, result.json)
        self.assertEqual(expected_status_code, result.status_code)
        get_mock.assert_called_once_with('status/runtimeinfo', None)

    def test_targets(self):
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": {
                "activeTargets": [
                    {
                        "discoveredLabels": {
                            "__address__": "localhost:9090",
                            "__metrics_path__": "/metrics",
                            "__scheme__": "http",
                            "__scrape_interval__": "15s",
                            "__scrape_timeout__": "10s",
                            "job": "prometheus"
                            },
                        "labels": {
                            "instance": "localhost:9090",
                            "job": "prometheus"
                            },
                        "scrapePool": "prometheus",
                        "scrapeUrl": "http://localhost:9090/metrics",
                        "globalUrl": "http://localhost.locald:9090/metrics",
                        "lastError": "",
                        "lastScrape": "2025-06-06T13:16:11.554579236Z",
                        "lastScrapeDuration": 0.006158981,
                        "health": "up",
                        "scrapeInterval": "15s",
                        "scrapeTimeout": "10s"
                        }
                    ],
                "droppedTargets": [],
                "droppedTargetCounts": None
                }
            }
        params = {'state': 'running'}

        with mock.patch.object(prometheus_client.PrometheusAPIClient, '_get',
                               return_value=returned_from_prometheus
                               ) as get_mock:
            result = self.get_json('/targets', **params,
                                   headers=self.admin_auth_headers,
                                   status=expected_status_code)

        self.assertEqual(returned_from_prometheus, result.json)
        self.assertEqual(expected_status_code, result.status_code)
        get_mock.assert_called_once_with('targets', params)


class CoreEndpointsErrorCommonTests():
    def test_label(self):
        with base.quiet_expected_exception():
            result = self.get_json('/label/name/values',
                                   headers=self.admin_auth_headers,
                                   status=self.expected_status_code,
                                   expect_errors=True)
        self.assertEqual(self.expected_status_code, result.status_code)

    def test_labels(self):
        with base.quiet_expected_exception():
            result = self.get_json('/labels',
                                   headers=self.admin_auth_headers,
                                   status=self.expected_status_code,
                                   expect_errors=True)

        self.assertEqual(self.expected_status_code, result.status_code)

    def test_query(self):
        with base.quiet_expected_exception():
            result = self.get_json('/query', query="some_query{l='lvalue'}",
                                   headers=self.admin_auth_headers,
                                   status=self.expected_status_code,
                                   expect_errors=True)

        self.assertEqual(self.expected_status_code, result.status_code)

    def test_series(self):
        args = {"match[]": ["metric_name1", "metric_name2"]}
        with base.quiet_expected_exception():
            result = self.get_json('/series', **args,
                                   headers=self.admin_auth_headers,
                                   status=self.expected_status_code,
                                   expect_errors=True)

        self.assertEqual(self.expected_status_code, result.status_code)

    def test_status(self):
        with base.quiet_expected_exception():
            result = self.get_json('/status/runtimeinfo',
                                   headers=self.admin_auth_headers,
                                   status=self.expected_status_code,
                                   expect_errors=True)

        self.assertEqual(self.expected_status_code, result.status_code)

    def test_targets(self):
        with base.quiet_expected_exception():
            result = self.get_json('/targets/somestate',
                                   headers=self.admin_auth_headers,
                                   status=self.expected_status_code,
                                   expect_errors=True)

        self.assertEqual(self.expected_status_code, result.status_code)


class TestCoreEndpointsServerSideError(
    base.TestCase,
    CoreEndpointsErrorCommonTests
):
    def setUp(self):
        super().setUp()
        self.expected_status_code = 508

        exception = prometheus_client.PrometheusAPIClientError(
            base.ErrorResponse(self.expected_status_code)
        )
        self.mock_get = mock.patch.object(
            prometheus_client.PrometheusAPIClient,
            '_get',
            side_effect=exception
        )
        self.mock_get.start()

    def tearDown(self):
        self.mock_get.stop()
        super().tearDown()


class TestCoreEndpointsClientSideError(
    base.TestCase,
    CoreEndpointsErrorCommonTests
):
    def setUp(self):
        super().setUp()
        self.expected_status_code = 418

        exception = prometheus_client.PrometheusAPIClientError(
            base.ErrorResponse(self.expected_status_code)
        )
        self.mock_get = mock.patch.object(
            prometheus_client.PrometheusAPIClient,
            '_get',
            side_effect=exception
        )
        self.mock_get.start()

    def tearDown(self):
        self.mock_get.stop()
        super().tearDown()


class TestCoreEndpointsUnexpectedStatusCodeError(
    base.TestCase,
    CoreEndpointsErrorCommonTests
):
    def setUp(self):
        super().setUp()
        self.expected_status_code = 501

        exception = prometheus_client.PrometheusAPIClientError(
            base.ErrorResponse(102)
        )
        self.mock_get = mock.patch.object(
            prometheus_client.PrometheusAPIClient,
            '_get',
            side_effect=exception
        )
        self.mock_get.start()

    def tearDown(self):
        self.mock_get.stop()
        super().tearDown()



#
# Copyright 2025 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oslo_log import log
import pecan
from webob import exc
from wsme import exc as wsme_exc

from aetos.controllers.api.v1 import base
from aetos import rbac

LOG = log.getLogger(__name__)


class LabelsController(base.Base):
    @pecan.expose(content_type='application/json')
    def get(self, *args, **kwargs):
        """Labels endpoint"""
        target = {"project_id": pecan.request.headers.get('X-Project-Id')}
        try:
            rbac.enforce('labels:all_projects', pecan.request.headers,
                         pecan.request.enforcer, target)
            privileged = True
            LOG.debug(
                "Received a high privilege request for the labels endpoint"
            )
        except exc.HTTPForbidden:
            rbac.enforce('labels', pecan.request.headers,
                         pecan.request.enforcer, target)
            privileged = False
            LOG.debug(
                "Received a low privilege request for the labels endpoint"
            )

        status_code = 200

        if len(args) != 0:
            pecan.response.status = 404
            return json.dumps("page not found")

        matches = kwargs.get('match[]', [])

        self.create_prometheus_client(pecan.request.cfg)

        processed_matches = self.process_matches(
            matches, privileged, target['project_id'], pecan.request.cfg
        )

        LOG.debug("Matches sent to prometheus: %s", str(processed_matches))
        try:
            result = self.prometheus_get(
                "labels", {"match[]": processed_matches}
            )
        except wsme_exc.ClientSideError as e:
            status_code = e.code
            result = e.msg

        LOG.debug("Data received from prometheus: %s", str(result))

        pecan.response.status = status_code
        return json.dumps(result)



# Copyright 2010-2011 OpenStack Foundation
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock
import webtest
import wsme

from oslo_config import fixture as fixture_config

from aetos import app
from aetos import service
from aetos.tests import base


def quiet_expected_exception():
    # NOTE(jwysogla): WSME will output a traceback to stdout whenever an
    # exception resulting in 5** http status code is raised. There are a
    # few tests, which expect various 5** statuses. Use this to get rid of
    # the unnecesssary tracebacks in the test output.
    r = dict(faultcode=500,
             faultstring='faultstring',
             debuginfo=None)
    return mock.patch.object(wsme.api, 'format_exception',
                             return_value=r)


class TestCase(base.TestCase):

    """Test case base class for all functional tests."""
    PATH_PREFIX = '/api/v1'

    def setUp(self):
        super().setUp()
        self.project_id = 'project1'
        self.admin_auth_headers = {'X-User-Id': 'admin_user',
                                   'X-Project-Id': self.project_id,
                                   'X-Roles': 'admin'}

        self.reader_auth_headers = {'X-User-Id': 'reader_user',
                                    'X-Project-Id': self.project_id,
                                    'X-Roles': 'reader'}

        conf = service.prepare_service(argv=[], config_files=[])
        self.CONF = self.useFixture(fixture_config.Config(conf)).conf
        self.CONF.set_override('auth_mode', None, group=None)

        self.app = webtest.TestApp(app.load_app(self.CONF))

    def tearDown(self):
        super().tearDown()

    def post_json(self, path, params, expect_errors=False, headers=None,
                  method="post", extra_environ=None, status=None):
        """Sends simulated HTTP POST request to Pecan test app.

        :param path: url path of target service
        :param params: content for wsgi.input of request
        :param expect_errors: boolean value whether an error is expected based
                              on request
        :param headers: A dictionary of headers to send along with the request
        :param method: Request method type. Appropriate method function call
                       should be used rather than passing attribute in.
        :param extra_environ: A dictionary of environ variables to send along
                              with the request
        :param status: Expected status code of response
        """
        full_path = self.PATH_PREFIX + path
        response = getattr(self.app, "%s_json" % method)(
            str(full_path),
            params=params,
            headers=headers,
            status=status,
            extra_environ=extra_environ,
            expect_errors=expect_errors
        )
        return response

    def get_json(self, path, expect_errors=False, headers=None,
                 extra_environ=None, status=None, **params):
        """Sends simulated HTTP GET request to Pecan test app.

        :param path: url path of target service
        :param expect_errors: boolean value whether an error is expected based
                              on request
        :param headers: A dictionary of headers to send along with the request
        :param extra_environ: A dictionary of environ variables to send along
                              with the request
        :param status: Expected status code of response
        :param params: content for wsgi.input of request
        """
        full_path = self.PATH_PREFIX + path
        response = self.app.get(full_path,
                                params=params,
                                headers=headers,
                                extra_environ=extra_environ,
                                expect_errors=expect_errors,
                                status=status)
        return response


class ErrorResponse():
    def __init__(self, status_code):
        self.status_code = status_code

    def json(self):
        return {"status": "error", "error": "test_error"}



#
# Copyright 2025 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from pecan import rest
from wsme import exc

from observabilityclient import prometheus_client
from observabilityclient import rbac as obsc_rbac
from oslo_config import cfg
from oslo_log import log
from oslo_utils import netutils

LOG = log.getLogger(__name__)


PROMETHEUS_OPTS = [
    cfg.StrOpt(
        'host',
        default="localhost",
        help="The host of Prometheus"),
    cfg.PortOpt(
        'port',
        default=9090,
        help="The port of Prometheus"),
    cfg.StrOpt(
        'cert_file',
        help="Path to a client cert for establishing TLS connections to "
             "Prometheus."
        ),
    cfg.StrOpt(
        'key_file',
        help="Path to a client key for establishing TLS connections to "
             "Prometheus."
        ),
    cfg.StrOpt(
        'ca_file',
        help="Path to a CA cert for establishing TLS connections to "
             "Prometheus. This is optional, if this isn't set, then "
             "default system CA certificates will be used."
        ),
    cfg.BoolOpt(
        'use_tls',
        default=False,
        help="Whether TLS should be used when connecting to Prometheus."
        ),
    cfg.StrOpt(
        'project_label_name',
        default="project",
        help="Label name used to store project IDs in Prometheus. "
             "This will be used for distinguishing between projects "
             "for multi-tenancy reasons."
        )
]


class ServerSideError(exc.ClientSideError):
    def __init__(self, error, status_code=500):
        super().__init__(error, status_code, 'Server')


class Base(rest.RestController):
    def create_prometheus_client(self, conf):
        host = netutils.escape_ipv6(conf.prometheus.host)
        port = conf.prometheus.port
        cert_file = conf.prometheus.cert_file
        key_file = conf.prometheus.key_file
        ca_file = conf.prometheus.ca_file
        use_tls = conf.prometheus.use_tls

        if (ca_file or key_file or cert_file) and not use_tls:
            LOG.warning("CA/Cert/Key file specified but TLS disabled - "
                        "Ca/Cert/Key file will be ignored")

        url = f"{host}:{port}"
        self.prometheus_client = prometheus_client.PrometheusAPIClient(url)

        if use_tls:
            if ca_file:
                LOG.debug("TLS for Prometheus connection enabled with CA "
                          "file: %s", ca_file)
                self.prometheus_client.set_ca_cert(ca_file)
            else:
                LOG.debug("TLS for Prometheus connection enabled with system "
                          "default CA certificates")
                self.prometheus_client.set_ca_cert(True)

            if cert_file and key_file:
                self.prometheus_client.set_client_cert(cert_file, key_file)
            elif cert_file or key_file:
                LOG.warning("Only one of cert_file and key_file is set. "
                            "Set the two options or both are ignored.")

        super(object, self).__init__()

    def process_matches(self, matches, privileged, project_id, conf):
        # Ensure matches is always a list
        if not isinstance(matches, list):
            matches = [matches]

        # If user has high privileges, return matches as-is
        if privileged:
            return matches

        promQLRbac = obsc_rbac.PromQLRbac(
            self.prometheus_client, project_id,
            conf.prometheus.project_label_name)

        if matches == []:
            return promQLRbac.append_rbac_labels('')

        # Apply RBAC modification to each match
        metric_names = self.prometheus_client.label_values("__name__")
        modified_matches = []
        for match in matches:
            modified_matches.append(
                promQLRbac.modify_query(match, metric_names=metric_names))

        return modified_matches

    @staticmethod
    def _get_correct_exception(e):
        if e.resp.status_code // 100 == 4:
            return exc.ClientSideError(str(e), e.resp.status_code)
        elif e.resp.status_code // 100 == 5:
            return ServerSideError(str(e), e.resp.status_code)
        else:
            return ServerSideError(
                (f'Aetos received an unexpected status code from '
                 f'prometheus: "{e}"'),
                501
            )

    def prometheus_get(self, endpoint, params=None):
        try:
            return self.prometheus_client._get(endpoint, params)
        except prometheus_client.PrometheusAPIClientError as e:
            if e.resp.status_code != 204:
                # NOTE(jwysogla): Prometheus can return empty responses, which
                # is expected, so ignore the 204 status code and reraise on
                # all others.
                raise Base._get_correct_exception(e)

    def prometheus_post(self, endpoint, params=None):
        try:
            return self.prometheus_client._post(endpoint, params)
        except prometheus_client.PrometheusAPIClientError as e:
            if e.resp.status_code != 204:
                # NOTE(jwysogla): Prometheus can return empty responses, which
                # is expected, so ignore the 204 status code and reraise on
                # all others.
                raise Base._get_correct_exception(e)



#
# Copyright 2025 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from observabilityclient import rbac as obsc_rbac
from oslo_log import log
import pecan
from webob import exc
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from aetos.controllers.api.v1 import base
from aetos import rbac

LOG = log.getLogger(__name__)


class QueryController(base.Base):
    @wsme_pecan.wsexpose(wtypes.text, wtypes.text, wtypes.text, wtypes.text)
    def get(self, query, time=None, timeout=None):
        """Query endpoint"""
        target = {"project_id": pecan.request.headers.get('X-Project-Id')}
        try:
            rbac.enforce('query:all_projects', pecan.request.headers,
                         pecan.request.enforcer, target)
            privileged = True
            LOG.debug(
                "Received a high privilege request for the query endpoint"
            )
        except exc.HTTPForbidden:
            rbac.enforce('query', pecan.request.headers,
                         pecan.request.enforcer, target)
            privileged = False
            LOG.debug(
                "Received a low privilege request for the query endpoint"
            )

        self.create_prometheus_client(pecan.request.cfg)

        modified_query = ""
        if privileged:
            modified_query = query
        else:
            promQLRbac = obsc_rbac.PromQLRbac(
                self.prometheus_client,
                target['project_id'],
                pecan.request.cfg.prometheus.project_label_name
            )
            modified_query = promQLRbac.modify_query(query)

        LOG.debug("Unmodified query: %s", query)
        LOG.debug("Query sent to prometheus: %s", modified_query)

        params = {"query": modified_query, "time": time, "timeout": timeout}
        result = self.prometheus_get("query", params)

        LOG.debug("Data received from prometheus: %s", str(result))
        return result



#
# Copyright 2025 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oslo_log import log
import pecan
from wsme import exc

from aetos.controllers.api.v1 import base
from aetos import rbac

LOG = log.getLogger(__name__)


class DeleteSeriesController(base.Base):
    # NOTE(jwysogla): the delete_series/ endpoint expects a `match[]` argument,
    # which is making the use of wsexpose difficult, so a plain
    # pecan.expose is used instead, with handling of the arguments
    # as a dictionary inside the function.
    @pecan.expose(content_type='application/json')
    def post(self, *args, **kwargs):
        """Delete_series endpoint"""
        # TODO(jwysogla):
        # - handle unknown, missing and optional parameters
        # - handle unsuccessful calls to prometheus

        target = {"project_id": pecan.request.headers.get('X-Project-Id')}
        rbac.enforce('admin_delete_metrics', pecan.request.headers,
                     pecan.request.enforcer, target)

        if len(args) != 0:
            pecan.response.status = 404
            return json.dumps("page not found")

        self.create_prometheus_client(pecan.request.cfg)
        matches = kwargs.get('match[]', [])
        start = kwargs.get('start', None)
        end = kwargs.get('end', None)
        try:
            self.prometheus_post("admin/tsdb/delete_series",
                                 {"match[]": matches,
                                  "start": start,
                                  "end": end})
        except exc.ClientSideError as e:
            # NOTE(jwysogla): We need a special handling of the exceptions,
            # because we don't use wsexpose as with most of other endpoints.
            pecan.response.status = e.code
            return e.msg
        # On success don't return anything



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_admin_endpoints
----------------------------------

Tests for endpoints under /api/v1/admin/tsdb
"""

from unittest import mock

from observabilityclient import prometheus_client

from aetos.tests.functional import base


class TestAdminEndpointsAsReader(base.TestCase):
    def setUp(self):
        super().setUp()
        self.expected_status_code = 403
        self.expected_fault_string = "RBAC Authorization Failed"

    def test_snapshot(self):
        result = self.post_json('/admin/tsdb/snapshot', {},
                                headers=self.reader_auth_headers,
                                status=self.expected_status_code,
                                expect_errors=True)

        self.assertEqual(self.expected_status_code, result.status_code)
        self.assertEqual(self.expected_fault_string,
                         result.json['error_message']['faultstring'])

    def test_delete_series(self):
        params = {"match[]": ["metric_name1", "metric_name2"]}
        result = self.post_json('/admin/tsdb/delete_series', params,
                                headers=self.reader_auth_headers,
                                status=self.expected_status_code,
                                expect_errors=True)

        self.assertEqual(self.expected_status_code, result.status_code)
        # NOTE(jwysogla): the delete_series doesn't use wsme, so the
        # error message is slightly differently formatted, but the same
        # status code is still returned and the meaning of the error message
        # is also the same. But this is the reason why we're using assertIn
        # instead of assertEqual here.
        self.assertIn(self.expected_fault_string, result.json['error_message'])

    def test_clean_tombstones(self):
        result = self.post_json('/admin/tsdb/clean_tombstones', {},
                                headers=self.reader_auth_headers,
                                status=self.expected_status_code,
                                expect_errors=True)

        self.assertEqual(self.expected_status_code, result.status_code)
        self.assertEqual(self.expected_fault_string,
                         result.json['error_message']['faultstring'])


class TestAdminEndpointsAsAdmin(base.TestCase):
    def test_snapshot(self):
        expected_status_code = 200
        returned_from_prometheus = {
            "status": "success",
            "data": {
                "name": 'somefilename'
            }
        }

        with mock.patch.object(
            prometheus_client.PrometheusAPIClient,
            '_post',
            return_value=returned_from_prometheus
        ):
            result = self.post_json('/admin/tsdb/snapshot', {},
                                    headers=self.admin_auth_headers,
                                    status=expected_status_code)

        self.assertEqual(expected_status_code, result.status_code)
        self.assertEqual(returned_from_prometheus, result.json)

    def test_delete_series(self):
        expected_status_code = 204
        params = {"match[]": ["metric_name1", "metric_name2"]}
        with mock.patch.object(
            prometheus_client.PrometheusAPIClient,
            '_post',
            return_value={}
        ):
            result = self.post_json('/admin/tsdb/delete_series', params,
                                    headers=self.admin_auth_headers,
                                    status=expected_status_code)

        self.assertEqual(expected_status_code, result.status_code)

    def test_clean_tombstones(self):
        expected_status_code = 204
        with mock.patch.object(
            prometheus_client.PrometheusAPIClient,
            '_post',
            return_value={}
        ):
            result = self.post_json('/admin/tsdb/clean_tombstones', {},
                                    headers=self.admin_auth_headers,
                                    status=expected_status_code)

        self.assertEqual(expected_status_code, result.status_code)


class AdminEndpointsErrorCommonTests():
    def test_delete_series(self):
        params = {"match[]": ["metric_name1", "metric_name2"]}
        with base.quiet_expected_exception():
            result = self.post_json('/admin/tsdb/delete_series', params,
                                    headers=self.admin_auth_headers,
                                    status=self.expected_status_code,
                                    expect_errors=True)

        self.assertEqual(self.expected_status_code, result.status_code)

    def test_snapshot(self):
        with base.quiet_expected_exception():
            result = self.post_json('/admin/tsdb/snapshot', {},
                                    headers=self.admin_auth_headers,
                                    status=self.expected_status_code,
                                    expect_errors=True)

        self.assertEqual(self.expected_status_code, result.status_code)

    def test_clean_tombstones(self):
        with base.quiet_expected_exception():
            result = self.post_json('/admin/tsdb/clean_tombstones', {},
                                    headers=self.admin_auth_headers,
                                    status=self.expected_status_code,
                                    expect_errors=True)

        self.assertEqual(self.expected_status_code, result.status_code)


class TestAdminEndpointsServerSideError(
    base.TestCase,
    AdminEndpointsErrorCommonTests
):
    def setUp(self):
        super().setUp()
        self.expected_status_code = 508

        exception = prometheus_client.PrometheusAPIClientError(
            base.ErrorResponse(self.expected_status_code)
        )
        self.mock_post = mock.patch.object(
            prometheus_client.PrometheusAPIClient,
            '_post',
            side_effect=exception
        )
        self.mock_post.start()

    def tearDown(self):
        self.mock_post.stop()
        super().tearDown()


class TestAdminEndpointsClientSideError(
    base.TestCase,
    AdminEndpointsErrorCommonTests
):
    def setUp(self):
        super().setUp()
        self.expected_status_code = 418

        exception = prometheus_client.PrometheusAPIClientError(
            base.ErrorResponse(self.expected_status_code)
        )
        self.mock_post = mock.patch.object(
            prometheus_client.PrometheusAPIClient,
            '_post',
            side_effect=exception
        )
        self.mock_post.start()

    def tearDown(self):
        self.mock_post.stop()
        super().tearDown()


class TestAdminEndpointsUnexpectedStatusCodeError(
    base.TestCase,
    AdminEndpointsErrorCommonTests
):
    def setUp(self):
        super().setUp()
        self.expected_status_code = 501

        exception = prometheus_client.PrometheusAPIClientError(
            base.ErrorResponse(102)
        )
        self.mock_post = mock.patch.object(
            prometheus_client.PrometheusAPIClient,
            '_post',
            side_effect=exception
        )
        self.mock_post.start()

    def tearDown(self):
        self.mock_post.stop()
        super().tearDown()



#
# Copyright 2025 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log
import pecan
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from aetos.controllers.api.v1 import base
from aetos import rbac

LOG = log.getLogger(__name__)


class TargetsController(base.Base):
    @wsme_pecan.wsexpose(wtypes.text, wtypes.text)
    def get(self, state):
        """Targets endpoint"""
        target = {"project_id": pecan.request.headers.get('X-Project-Id')}
        rbac.enforce('targets', pecan.request.headers,
                     pecan.request.enforcer, target)

        self.create_prometheus_client(pecan.request.cfg)
        result = self.prometheus_get("targets", {'state': state})
        LOG.debug("Data received from prometheus: %s", str(result))
        return result
